
#include"rfc/rfc.h"
#include"lice/lice/lice.h"
#include "ucdl.h"

#include<math.h>

#include"resource.h"

COLORREF colCitrus=RGB(230,119,11);

LICE_IBitmap *currentImage;
LICE_SysBitmap* undoImage;

KString currentDirectory;


class LimeDebugViewer : public KToolWindow, public KButtonListener, public KMenuItemListener
{
	KOverlappedWindow* parent;
	KListBox listBox;
	KMenuBar mBar;
	KMenu mFile;
	KMenuItem mClearLogs;
	KFont* listBoxFont;

	HBRUSH hbrListBox;

	static const int MENU_IMAGE_SIZE=20;
	static const int MENU_HEIGHT=22;

public:
	LimeDebugViewer(KOverlappedWindow* parent)
	{
		this->parent=parent;
		SetText("Debug Logs Viewer");
		SetSize(400,350);
		SetStyle(WS_SIZEBOX|WS_OVERLAPPED|WS_SYSMENU);
		CenterScreen();

		hbrListBox=CreateSolidBrush(colCitrus);
	}

	~LimeDebugViewer()
	{
		DeleteObject(hbrListBox);
		delete listBoxFont;
	}

	void CreateSubComponents()
	{
		listBox.SetSize(392,340);
		listBox.SetPosition(0,0);
		listBox.SetStyle(listBox.GetStyle()|LBS_NOSEL);
		AddComponent(&listBox);

		mClearLogs.SetText("Clear All");
		mClearLogs.SetListener(this);

		mFile.AddMenuItem(&mClearLogs);

		mBar.AddMenu("File",&mFile);
		mBar.AddToWindow(this);

		ModifyMenuW(mFile.GetMenuHandle(),mClearLogs.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mClearLogs.GetItemID(),(LPCWSTR)mClearLogs.GetItemID());

		listBoxFont=new KFont("Lucida Console",13,true);
		listBox.SetFont(listBoxFont);
	}

	BEGIN_KMSG_HANDLER
		ON_KMSG(WM_CTLCOLORLISTBOX,OnListBoxColor)
		ON_KMSG(WM_SIZE,OnResized) 
		ON_KMSG(WM_SIZING,OnResizing)
		ON_KMSG(WM_MEASUREITEM,OnWMMeasureItem)
		ON_KMSG(WM_DRAWITEM,OnDrawItem)
	END_KMSG_HANDLER(KToolWindow)

	LRESULT OnResizing(WPARAM wParam,LPARAM lParam)
	{
		const int MIN_WIDTH=400;
		const int MIN_HEIGHT=350;

		LPRECT lprc;
		lprc = (LPRECT) lParam;

		if((lprc->right-lprc->left)<MIN_WIDTH){
			lprc->right=lprc->left+MIN_WIDTH;
		}
		if((lprc->bottom-lprc->top)<MIN_HEIGHT){
			lprc->bottom=lprc->top+MIN_HEIGHT;
		}

		return 1;
	}

	LRESULT OnListBoxColor(WPARAM wParam,LPARAM lParam)
	{
		HDC hdcList = (HDC)wParam;

		SetTextColor(hdcList, RGB(255,255,255));
		SetBkColor(hdcList, colCitrus);

		return (LRESULT)hbrListBox;
	}

	LRESULT OnResized(WPARAM wParam,LPARAM lParam)
	{
		int nWidth = LOWORD(lParam);
		int nHeight = HIWORD(lParam);

		listBox.SetSize(nWidth-2,nHeight);

		return 0;
	}

	LRESULT OnDrawItem(WPARAM wParam,LPARAM lParam)
	{
		if(wParam==0) // it's a menu
		{
			LPDRAWITEMSTRUCT dis=(LPDRAWITEMSTRUCT)lParam;

			FillRect(dis->hDC,&dis->rcItem,(HBRUSH)GetSysColorBrush(COLOR_MENU));

			KMenuItem *item=KPlatformUtil::GetInstance()->GetMenuItemByID((UINT)dis->itemData);

			if ( (dis->itemState & ODS_SELECTED) && (dis->itemAction & (ODA_SELECT | ODA_DRAWENTIRE)) )
			{
				HBRUSH hbr=CreateSolidBrush(colCitrus);
				FillRect(dis->hDC,&dis->rcItem,hbr);
				DeleteObject(hbr);

				COLORREF oldCol=SetTextColor(dis->hDC,RGB(255,255,255));
				int oldBk=SetBkMode(dis->hDC, TRANSPARENT); 
				dis->rcItem.left+=MENU_IMAGE_SIZE;
				DrawText(dis->hDC,item->GetText(),item->GetText().GetLength(),&dis->rcItem,DT_SINGLELINE|DT_VCENTER|DT_LEFT);
				dis->rcItem.left-=MENU_IMAGE_SIZE;		
				SetTextColor(dis->hDC,oldCol);
				SetBkMode(dis->hDC, oldBk); 
			}else
			{
				dis->rcItem.left+=MENU_IMAGE_SIZE;
				DrawText(dis->hDC,item->GetText(),item->GetText().GetLength(),&dis->rcItem,DT_SINGLELINE|DT_VCENTER|DT_LEFT);
				dis->rcItem.left-=MENU_IMAGE_SIZE;
			}
			return TRUE;
		}else
		{
			return KToolWindow::WindowProc(compHWND,WM_DRAWITEM,wParam,lParam);
		}
	}

	LRESULT OnWMMeasureItem(WPARAM wParam,LPARAM lParam)
	{
		if(wParam==0) // it's a menu
		{
			LPMEASUREITEMSTRUCT mis=(LPMEASUREITEMSTRUCT)lParam;
			mis->itemHeight=MENU_HEIGHT;
			HDC hdc=GetDC(compHWND);
			HGDIOBJ hfontOld = SelectObject(hdc,(HFONT)GetStockObject(DEFAULT_GUI_FONT));
			KMenuItem *item=KPlatformUtil::GetInstance()->GetMenuItemByID((UINT)mis->itemData);
			SIZE sz={0,0};
			GetTextExtentPoint32W(hdc,item->GetText(),item->GetText().GetLength(),&sz);
			mis->itemWidth=sz.cx+MENU_IMAGE_SIZE;
			SelectObject(hdc,hfontOld);
			ReleaseDC(compHWND,hdc);

			return 1;
		}else
		{
			return KToolWindow::WindowProc(compHWND,WM_MEASUREITEM,wParam,lParam);
		}
	}


	 void OnMenuItemPress(KMenuItem *menuItem)
	 {
		 if(menuItem==&mClearLogs)
		 {
			listBox.ClearList();
		 }
	 }

	void DebugOutString(char* text)
	{
		int len=(int)strlen(text);
		if(len>0)
		{
			//remove newlines...
			for(int i=0;i<len;i++)
			{
				if(text[i]=='\n')
				{
					text[i]=' ';
				}
			}

			listBox.AddItem(text);
		}
	}

	void DebugOutInt(int num)
	{
		KString txt(num,10);
		listBox.AddItem(txt);
	}

	void OnClose()
	{
		SetVisible(false);
	}
};

LimeDebugViewer* limeDebugViewer=0;

//==========script apis============================

__declspec(dllexport) void DrawPNGImage(char* file,int x,int y,int w,int h,float alpha)
{
	LICE_IBitmap *img=LICE_LoadPNG(file);
	if(img)
	{
		LICE_ScaledBlit(currentImage,img,x,y,w,h,0,0,img->getWidth(),img->getHeight(),alpha,LICE_BLIT_USE_ALPHA);
		delete img;
	}
}

__declspec(dllexport) void LineTo(int x1, int y1, int x2, int y2, int col, float alpha)
{
	LICE_Line(currentImage,x1,y1,x2,y2,col,alpha,LICE_BLIT_MODE_COPY,true);
}

__declspec(dllexport) void Blur(int x,int y,int w,int h)
{
	LICE_Blur(currentImage,currentImage,x,y,x,y,w,h);
}

__declspec(dllexport) void FillConvexPolygon(int* x, int* y, int npoints, int col, float alpha)
{
	LICE_FillConvexPolygon(currentImage,x,y,npoints,col,alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) void FillCircle(float cx, float cy, float r, int col, float alpha)
{
	LICE_FillCircle(currentImage,cx, cy, r, col, alpha,LICE_BLIT_MODE_COPY,true);
}

__declspec(dllexport) void DrawCircle(float cx, float cy, float r, int col, float alpha)
{
	LICE_Circle(currentImage,cx,cy,r,col,alpha,LICE_BLIT_MODE_COPY,true);
}

__declspec(dllexport) void DrawArc(float cx,float cy,float r,float minAngle,float maxAngle,int col,float alpha)
{
	LICE_Arc(currentImage,cx,cy,r,minAngle,maxAngle,col,alpha,LICE_BLIT_MODE_COPY,true);
}

__declspec(dllexport) void FillTriangle(int x1, int y1, int x2, int y2, int x3, int y3, int col, float alpha)
{
	LICE_FillTriangle(currentImage,x1,y1,x2,y2,x3,y3,col,alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) void FillRectangle(int x,int y,int w,int h,int col,float alpha)
{
	LICE_FillRect(currentImage,x,y,w,h,col,alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) void DrawRectangle(int x, int y, int w, int h, int col, float alpha)
{
	LICE_DrawRect(currentImage, x, y, w, h, col, alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) int GetPixel(int x,int y)
{
	return LICE_GetPixel(currentImage,x,y);
}

__declspec(dllexport) void SetPixel(int x,int y,int col,float alpha)
{
	LICE_PutPixel(currentImage,x,y,col,alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) int GetWidth()
{
 return currentImage->getWidth();
}

__declspec(dllexport) int GetHeight()
{
 return currentImage->getHeight();
}

__declspec(dllexport) void DebugOutString(char* text)
{
	limeDebugViewer->DebugOutString(text);
}

__declspec(dllexport) void DebugOutInt(int num)
{
	limeDebugViewer->DebugOutInt(num);
}

__declspec(dllexport) void* LimeAlloc(int size)
{
	return malloc(size);
}

__declspec(dllexport) void LimeFree(void* buff)
{
	free(buff);
}

//================================================


class LimeTextArea : public KTextArea
{
public:

	BEGIN_KMSG_HANDLER
		ON_KMSG(WM_SETFOCUS ,OnSetFocus)
	END_KMSG_HANDLER(KTextArea)

	LRESULT OnSetFocus(WPARAM wParam,LPARAM lParam) // caret is solid ;-)
	{
		LRESULT retVal=KTextArea::WindowProc(compHWND,WM_SETFOCUS,wParam,lParam);
		CreateCaret(compHWND,0,4,13);
		ShowCaret(compHWND);
		return retVal;
	}

};

class LimeScriptEditor : public KToolWindow, public KButtonListener
{
	KOverlappedWindow* parent;
	LimeTextArea textArea;
	KButton btnBrowse,btnSave,btnExec,btnUndo;
	KFont* scriptFont;
	HBRUSH hbrEdit;

	
public:
	LimeScriptEditor(KOverlappedWindow* parent)
	{
		this->parent=parent;
		SetText("Lime Script Editor");
		SetSize(400,350);
		SetStyle(WS_SIZEBOX|WS_OVERLAPPED|WS_SYSMENU);
		CenterScreen();

		hbrEdit=CreateSolidBrush(colCitrus);
		
	}

	~LimeScriptEditor()
	{
		DeleteObject(hbrEdit);
		delete scriptFont;
	}

	BEGIN_KMSG_HANDLER
		ON_KMSG(WM_CTLCOLOREDIT,OnEditBoxColor)
		ON_KMSG(WM_SIZE,OnResized) 
		ON_KMSG(WM_SIZING,OnResizing)
	END_KMSG_HANDLER(KToolWindow)

	LRESULT OnEditBoxColor(WPARAM wParam,LPARAM lParam)
	{
		HDC hdcEdit = (HDC)wParam;

		SetTextColor(hdcEdit, RGB(255,255,255));
		SetBkColor(hdcEdit, colCitrus);

		return (LRESULT)hbrEdit;
	}

	LRESULT OnResized(WPARAM wParam,LPARAM lParam)
	{
		int nWidth = LOWORD(lParam);
		int nHeight = HIWORD(lParam);

		textArea.SetSize(nWidth-12,nHeight-60);

		btnBrowse.SetPosition(8,nHeight-36);
		btnSave.SetPosition(83,nHeight-36);
		btnExec.SetPosition(nWidth-89,nHeight-36);
		btnUndo.SetPosition(nWidth-175,nHeight-36);

		return 0;
	}

	LRESULT OnResizing(WPARAM wParam,LPARAM lParam)
	{
		const int MIN_WIDTH=400;
		const int MIN_HEIGHT=350;

		LPRECT lprc;
		lprc = (LPRECT) lParam;

		if((lprc->right-lprc->left)<MIN_WIDTH){
			lprc->right=lprc->left+MIN_WIDTH;
		}
		if((lprc->bottom-lprc->top)<MIN_HEIGHT){
			lprc->bottom=lprc->top+MIN_HEIGHT;
		}

		return 1;
	}

	void CreateSubComponents()
	{
		textArea.SetSize(380,265);
		textArea.SetPosition(8,10);
		AddComponent(&textArea);

		btnBrowse.SetText("Browse...");
		btnBrowse.SetPosition(8,290);
		btnBrowse.SetSize(70,25);
		btnBrowse.SetListener(this);
		AddComponent(&btnBrowse);

		btnSave.SetText("Save As...");
		btnSave.SetPosition(83,290);
		btnSave.SetSize(70,25);
		btnSave.SetListener(this);
		AddComponent(&btnSave);

		btnExec.SetText("Execute");
		btnExec.SetPosition(308,290);
		btnExec.SetSize(80,25);
		btnExec.SetListener(this);
		AddComponent(&btnExec);

		btnUndo.SetText("Undo");
		btnUndo.SetPosition(216,290);
		btnUndo.SetSize(80,25);
		btnUndo.SetListener(this);
		AddComponent(&btnUndo);

		scriptFont=new KFont("Lucida Console",13,true);
		textArea.SetFont(scriptFont);
	}

	 void OnButtonPress(KButton *button)
	 {
		if(button==&btnExec)
		{
			if( (currentImage!=0) && (!textArea.GetText().IsEmpty()) )
			{
				LICE_Copy(undoImage,currentImage);
				KString txt=textArea.GetText();
				int result=uc_exec((char*)(const char*)txt);
				if(result==0) // error in syntax!
				{
					char buff[256];
					uc_error(buff,256);
					limeDebugViewer->DebugOutString(buff);
				}
				parent->Repaint();
			}
		}else if(button==&btnBrowse)
		{
			KCommonDialogBox cb;
			KString fileName;
			if(cb.ShowOpenFileDialog(this,L"Open Script...",L"Script Files (*.lms)\0*.lms\0",&fileName))
			{
				HANDLE hFile=CreateFile(fileName,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
				DWORD fileSize=GetFileSize(hFile,0);
				if(fileSize)
				{
					char* buffer=(char*)malloc(fileSize);
					DWORD lpNumberOfBytesRead;
					ReadFile(hFile,buffer,fileSize,&lpNumberOfBytesRead,0);
					CloseHandle(hFile);
					
					textArea.SetText(KString(buffer));
					free(buffer);

					SetCurrentDirectoryW(currentDirectory);
				}
			}
		}else if(button==&btnSave)
		{
			if(!textArea.GetText().IsEmpty())
			{
				KCommonDialogBox cb;
				KString fileName;
				if(cb.ShowSaveFileDialog(this,L"Save Script...",L"Script Files (*.lms)\0*.lms\0",&fileName))
				{
					HANDLE hFile=CreateFile(fileName.Append(".lms"),GENERIC_WRITE,FILE_SHARE_READ,0,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
					KString txt=textArea.GetText();
					char* buffer=(char*)(const char*)txt;
					DWORD size=textArea.GetText().GetLength()+1;
					DWORD lpNumberOfBytesWritten;
					WriteFile(hFile,buffer,size,&lpNumberOfBytesWritten,0);
					CloseHandle(hFile);

					SetCurrentDirectoryW(currentDirectory);
				}
			}
		}else if(button==&btnUndo)
		{
			LICE_Copy(currentImage,undoImage);
			parent->Repaint();
		}

	 }

	void OnClose()
	{
		SetVisible(false);
	}
};

class LimeEditMainWindow : public KOverlappedWindow, public KMenuItemListener
{
	KMenuBar mBar;
	KMenu mFile,mEdit,mScript,mHelp;
	KMenuItem mOpen,mSave,mExit,mUndo,mScriptEdit,mDebugViewer,mManual,mAbout;
	LICE_SysBitmap *backBuffer;

	LimeScriptEditor* scriptEditor;
	KIcon limeIcon;

	static const int MENU_IMAGE_SIZE=20;
	static const int MENU_HEIGHT=22;

public:

	LimeEditMainWindow()
	{
		SetText("LimeEdit v1.0");
		SetSize(800,600);

		currentImage=0;
		backBuffer=new LICE_SysBitmap(compWidth ,compHeight);

		undoImage=new LICE_SysBitmap(10,10);

		limeIcon.LoadFromResource(IDI_ICON1);
	}

	~LimeEditMainWindow()
	{
		if(currentImage)
			delete currentImage;
		delete backBuffer;
		delete undoImage;
		delete scriptEditor;
		delete limeDebugViewer;
	}

	void CreateElements()
	{
		CreateComponent();
		SetIcon(&limeIcon);

		mOpen.SetText("Open Image...");
		mSave.SetText("Save Image As...");
		mExit.SetText("Exit");
		mUndo.SetText("Undo Last Action");
		mScriptEdit.SetText("Script Editor...");
		mDebugViewer.SetText("View Debug Logs...");
		mManual.SetText("LimeEdit Manual");
		mAbout.SetText("About LimeEdit...");

		mOpen.SetListener(this);
		mSave.SetListener(this);
		mExit.SetListener(this);
		mUndo.SetListener(this);
		mScriptEdit.SetListener(this);
		mDebugViewer.SetListener(this);
		mManual.SetListener(this);
		mAbout.SetListener(this);

		mFile.AddMenuItem(&mOpen);
		mFile.AddMenuItem(&mSave);
		mFile.AddSeperator();
		mFile.AddMenuItem(&mExit);

		mEdit.AddMenuItem(&mUndo);

		mScript.AddMenuItem(&mScriptEdit);
		mScript.AddSeperator();
		mScript.AddMenuItem(&mDebugViewer);

		mHelp.AddMenuItem(&mManual);
		mHelp.AddSeperator();
		mHelp.AddMenuItem(&mAbout);

		mBar.AddMenu("File",&mFile);
		mBar.AddMenu("Edit",&mEdit);
		mBar.AddMenu("Script",&mScript);
		mBar.AddMenu("Help",&mHelp);
		mBar.AddToWindow(this);

		ModifyMenuW(mFile.GetMenuHandle(),mOpen.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mOpen.GetItemID(),(LPCWSTR)mOpen.GetItemID());
		ModifyMenuW(mFile.GetMenuHandle(),mSave.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mSave.GetItemID(),(LPCWSTR)mSave.GetItemID());
		ModifyMenuW(mFile.GetMenuHandle(),mExit.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mExit.GetItemID(),(LPCWSTR)mExit.GetItemID());
		ModifyMenuW(mEdit.GetMenuHandle(),mUndo.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mUndo.GetItemID(),(LPCWSTR)mUndo.GetItemID());		
		ModifyMenuW(mScript.GetMenuHandle(),mScriptEdit.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mScriptEdit.GetItemID(),(LPCWSTR)mScriptEdit.GetItemID());
		ModifyMenuW(mScript.GetMenuHandle(),mDebugViewer.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mDebugViewer.GetItemID(),(LPCWSTR)mDebugViewer.GetItemID());		
		ModifyMenuW(mHelp.GetMenuHandle(),mManual.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mManual.GetItemID(),(LPCWSTR)mManual.GetItemID());
		ModifyMenuW(mHelp.GetMenuHandle(),mAbout.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mAbout.GetItemID(),(LPCWSTR)mAbout.GetItemID());
	
		scriptEditor=new LimeScriptEditor(this);
		scriptEditor->SetParentHWND(compHWND);
		scriptEditor->CreateComponent();
		scriptEditor->CreateSubComponents();

		limeDebugViewer=new LimeDebugViewer(this);
		limeDebugViewer->SetParentHWND(compHWND);
		limeDebugViewer->CreateComponent();
		limeDebugViewer->CreateSubComponents();
	}

	void OnMenuItemPress(KMenuItem *menuItem)
	{
		
		if(menuItem==&mOpen)
		{
			KCommonDialogBox cb;
			KString fileName;
			if(cb.ShowOpenFileDialog(this,L"Open Image...",L"Image Files (*.png)\0*.png\0",&fileName))
			{
				if(currentImage)
					delete currentImage;

				currentImage=LICE_LoadPNG(fileName);
				LICE_Copy(undoImage,currentImage);

				SetCurrentDirectoryW(currentDirectory);

				this->Repaint();
			}	
		}else if(menuItem==&mSave)
		{
			if(currentImage)
			{
				KCommonDialogBox cb;
				KString fileName;
				if(cb.ShowSaveFileDialog(this,L"Save Image As...",L"Image Files (*.png)\0*.png\0",&fileName))
				{
					LICE_WritePNG(fileName.Append(".png"),currentImage);

					SetCurrentDirectoryW(currentDirectory);
				}
			}
		}else if(menuItem==&mExit)
		{
			SendMessage(compHWND,WM_CLOSE,0,0);
		}else if(menuItem==&mUndo)
		{
			if(currentImage)
			{
				LICE_Copy(currentImage,undoImage);
				Repaint();
			}
		}else if(menuItem==&mScriptEdit)
		{	
			if(currentImage)
			{
				scriptEditor->SetVisible(true);
			}
		}else if(menuItem==&mDebugViewer)
		{
			if(currentImage)
			{
				limeDebugViewer->SetVisible(true);
			}
		}else if(menuItem==&mManual)
		{
			ShellExecuteW(compHWND,L"open",L"manual.txt",0,0,SW_SHOWNORMAL);
		}
		else if(menuItem==&mAbout)
		{
			MSGBOXPARAMS msgparams;
			ZeroMemory(&msgparams,sizeof(MSGBOXPARAMS));
			msgparams.cbSize=sizeof(MSGBOXPARAMS);
			msgparams.hwndOwner=compHWND;
			msgparams.hInstance=KPlatformUtil::GetInstance()->GetAppInstance();
			msgparams.dwStyle=MB_OK|MB_USERICON;
			msgparams.lpszIcon=MAKEINTRESOURCEW(IDI_ICON1);
			msgparams.lpszText=L"LimeEdit v1.0\nDeveloped by Ruchira Hasaranga\nhttp://hase85.wordpress.com\n\nCreated with:\n\tMicrosoft C++ Compiler v8.0\n\tLibPNG version 1.2.16 \n\tZLib version 1.2.3\n\tCockos WDL - LICE\n\tUnderC version 1.2.9w\n\tRFC Framework v0.1.8\n\nLICE, ZLib, LibPNG and UnderC belogs to their respective owners.\nThis program is freeware and released under the GNU General Public License.";
			msgparams.lpszCaption=L"About";
			MessageBoxIndirect(&msgparams);
		}
	}

	BEGIN_KMSG_HANDLER
		ON_KMSG(WM_PAINT,OnPaint)
		ON_KMSG(WM_ERASEBKGND,OnEraseBack)
		ON_KMSG(WM_SIZING,OnResizing)
		ON_KMSG(WM_MEASUREITEM,OnWMMeasureItem)
		ON_KMSG(WM_DRAWITEM,OnDrawItem)
	END_KMSG_HANDLER(KOverlappedWindow)

	LRESULT OnDrawItem(WPARAM wParam,LPARAM lParam)
	{
		if(wParam==0) // it's a menu
		{
			LPDRAWITEMSTRUCT dis=(LPDRAWITEMSTRUCT)lParam;

			FillRect(dis->hDC,&dis->rcItem,(HBRUSH)GetSysColorBrush(COLOR_MENU));

			KMenuItem *item=KPlatformUtil::GetInstance()->GetMenuItemByID((UINT)dis->itemData);

			if ( (dis->itemState & ODS_SELECTED) && (dis->itemAction & (ODA_SELECT | ODA_DRAWENTIRE)) )
			{
				HBRUSH hbr=CreateSolidBrush(colCitrus);
				FillRect(dis->hDC,&dis->rcItem,hbr);
				DeleteObject(hbr);

				COLORREF oldCol=SetTextColor(dis->hDC,RGB(255,255,255));
				int oldBk=SetBkMode(dis->hDC, TRANSPARENT); 
				dis->rcItem.left+=MENU_IMAGE_SIZE;
				DrawText(dis->hDC,item->GetText(),item->GetText().GetLength(),&dis->rcItem,DT_SINGLELINE|DT_VCENTER|DT_LEFT);
				dis->rcItem.left-=MENU_IMAGE_SIZE;		
				SetTextColor(dis->hDC,oldCol);
				SetBkMode(dis->hDC, oldBk); 
			}else
			{
				dis->rcItem.left+=MENU_IMAGE_SIZE;
				DrawText(dis->hDC,item->GetText(),item->GetText().GetLength(),&dis->rcItem,DT_SINGLELINE|DT_VCENTER|DT_LEFT);
				dis->rcItem.left-=MENU_IMAGE_SIZE;
			}
			return TRUE;
		}else
		{
			return KOverlappedWindow::WindowProc(compHWND,WM_DRAWITEM,wParam,lParam);
		}
	}

	LRESULT OnWMMeasureItem(WPARAM wParam,LPARAM lParam)
	{
		if(wParam==0) // it's a menu
		{
			LPMEASUREITEMSTRUCT mis=(LPMEASUREITEMSTRUCT)lParam;
			mis->itemHeight=MENU_HEIGHT;
			HDC hdc=GetDC(compHWND);
			HGDIOBJ hfontOld = SelectObject(hdc,(HFONT)GetStockObject(DEFAULT_GUI_FONT));
			KMenuItem *item=KPlatformUtil::GetInstance()->GetMenuItemByID((UINT)mis->itemData);
			SIZE sz={0,0};
			GetTextExtentPoint32W(hdc,item->GetText(),item->GetText().GetLength(),&sz);
			mis->itemWidth=sz.cx+MENU_IMAGE_SIZE;
			SelectObject(hdc,hfontOld);
			ReleaseDC(compHWND,hdc);

			return 1;
		}else
		{
			return KOverlappedWindow::WindowProc(compHWND,WM_MEASUREITEM,wParam,lParam);
		}
	}

	LRESULT OnResizing(WPARAM wParam,LPARAM lParam)
	{
		const int MIN_WIDTH=400;
		const int MIN_HEIGHT=400;

		LPRECT lprc;
		lprc = (LPRECT) lParam;

		if((lprc->right-lprc->left)<MIN_WIDTH){
			lprc->right=lprc->left+MIN_WIDTH;
		}
		if((lprc->bottom-lprc->top)<MIN_HEIGHT){
			lprc->bottom=lprc->top+MIN_HEIGHT;
		}

		return 1;
	}

	LRESULT OnEraseBack(WPARAM wParam,LPARAM lParam)
	{
		return 1;
	}


	LRESULT OnPaint(WPARAM wParam,LPARAM lParam)
	{
		HDC hdc;
		PAINTSTRUCT ps;
		RECT rect;
		hdc=BeginPaint(compHWND,&ps);
		GetClientRect(compHWND,&rect);

		int clientWidth=rect.right-rect.left;
		int clientHeight=rect.bottom-rect.top;

		backBuffer->resize(clientWidth ,clientHeight);

		LICE_FillRect(backBuffer,0,0,clientWidth,clientHeight,LICE_RGBA(128,128,128,255));

		if(currentImage) // keep image ratio with window size!
		{
			int imgWidth=currentImage->getWidth();
			int imgHeight=currentImage->getHeight();
			if( (imgWidth>clientWidth) || (imgHeight>clientHeight) )
			{
				int newImgHeight,newImgWidth;

				if(imgHeight>imgWidth)
				{
					int i=20;
					while(true)
					{
						newImgHeight=clientHeight-i;
						newImgWidth=((float)newImgHeight/(float)imgHeight)*(float)imgWidth;	
						if( newImgWidth < (clientWidth-20) )
						{
							break;
						}
						i++;
					}
				}else
				{
					int i=20;
					while(true)
					{
						newImgWidth=clientWidth-i;
						newImgHeight=((float)newImgWidth/(float)imgWidth)*(float)imgHeight;	
						if( newImgHeight < (clientHeight-20) )
						{
							break;
						}
						i++;
					}
				}

				LICE_ScaledBlit(backBuffer,currentImage,(clientWidth-newImgWidth)/2,(clientHeight-newImgHeight)/2,newImgWidth,newImgHeight,0,0,imgWidth,imgHeight,1.0,LICE_BLIT_USE_ALPHA);

			}else
			{
				LICE_Blit(backBuffer,currentImage,(clientWidth-imgWidth)/2,(clientHeight-imgHeight)/2,0,0,imgWidth,imgHeight,1.0,LICE_BLIT_USE_ALPHA);
			}
		}
		
		BitBlt(hdc,0,0,clientWidth ,clientHeight,backBuffer->getDC(),0,0,SRCCOPY);	
		
		EndPaint(compHWND,&ps);

		return 0;
	}

};

class MyApp : public KApplication
{
public:
	int Main(KString **argv,int argc)
	{
		// setup path...
		static wchar_t path[MAX_PATH];
		GetModuleFileNameW(KPlatformUtil::GetInstance()->GetAppInstance(),path,MAX_PATH);

		wchar_t *p;
		for (p = path; *p; p++) { }	// find end
		for (; p > path && *p != '\\'; p--) { } // back up to last backslash
		*p = 0;	// kill it

		currentDirectory=path;

		uc_init(0,0);
		uc_include("lime.h");

		LimeEditMainWindow wnd;

		wnd.CenterScreen();
		wnd.SetVisible(true);
		wnd.CreateElements();

		DoMessagePump(false);

		uc_finis();

		return 0;
	}
};

START_RFC_APPLICATION(MyApp)