
/*
RFC - KString.cpp
Copyright (C) 2010 Ruchira Hasaranga

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.

*/

#include"KString.h"

const KString operator+ (const char* const string1,const KString& string2)
{
	KString s(string1);
	s = s+string2;
	return s;
}

const KString operator+ (const wchar_t* const string1,const KString& string2)
{
	KString s(string1);
	s = s+string2;
	return s;
}

const KString operator+ (const KString& string1,const KString& string2)
{
	KString s(string1);
	s = s+string2;
	return s;
}

KString::KString()
{
	a_text=0;
	w_text=0;
	count=0;
}

KString::KString(const KString& other)
{
	if(other.count)
	{
		a_text=(char*)malloc(other.count+1);
		strcpy(a_text,other.a_text);

		w_text=(wchar_t*)malloc((other.count+1)*sizeof(wchar_t));
		wcscpy(w_text,other.w_text);
	}else
	{
		a_text=0;
		w_text=0;
	}

	count=other.count;		
}

KString::KString(const char* const text,UINT codePage)
{
	if ( (text != 0) && (*text != 0) )
	{
		int length = (int)strlen(text);
		a_text=(char*)malloc(length+1);
		strcpy(a_text,text);

		count=length;

		length = MultiByteToWideChar(codePage, 0, text, -1, 0,0);
		if (length)
		{
			w_text = (wchar_t *)malloc(length*sizeof(wchar_t));
			if (MultiByteToWideChar(codePage, 0, text, -1, w_text,length))
			{
				return;
			}else
			{
				free(w_text);
				free(a_text);
			}
		}else
		{
			free(a_text);
		}
	}

	a_text=0;
	w_text=0;
	count=0;		
}

KString::KString(const wchar_t* const text,UINT codePage)
{
	if ( (text != 0) && (*text != 0) )
	{
		int length = (int)wcslen(text);
		w_text=(wchar_t*)malloc((length+1)*sizeof(wchar_t));
		wcscpy(w_text,text);

		count=length;

		length = WideCharToMultiByte(codePage,0, text, -1, 0,0,0,0);
		if (length)
		{
			a_text = (char*)malloc(length);
			if (WideCharToMultiByte(codePage,0, text, -1, a_text,length,0,0))
			{
				return;
			}else
			{
				free(w_text);
				free(a_text);
			}
		}else
		{
			free(w_text);
		}
	}

	a_text=0;
	w_text=0;
	count=0;		
}

KString::KString(const int value,const int radix)
{
	char text[33];
	_itoa(value,text,radix);

	int length = (int)strlen(text);
	a_text=(char*)malloc(length+1);
	strcpy(a_text,text);

	count=length;

	length = MultiByteToWideChar(CP_ACP, 0, text, -1, 0,0);
	if (length)
	{
		w_text = (wchar_t *)malloc(length*sizeof(wchar_t));
		if (!MultiByteToWideChar(CP_ACP, 0, text, -1, w_text,length))			
		{
			free(w_text);
			free(a_text);
			a_text=0;
			w_text=0;
			count=0;
		}
	}else
	{
		free(a_text);
		a_text=0;
		w_text=0;
		count=0;
	}
}

const KString& KString::operator= (const KString& other)
{
	if(other.count)
	{
		if(count)
		{
			free(a_text);
			free(w_text);
		}
		a_text=(char*)malloc(other.count+1);
		w_text=(wchar_t*)malloc((other.count+1)*sizeof(wchar_t));
		strcpy(a_text,other.a_text);
		wcscpy(w_text,other.w_text);
		count=other.count;
	}else
	{
		if(count)
		{
			free(a_text);
			free(w_text);
			a_text=0;
			w_text=0;
			count=0;
		}

	}
	return *this;
}

const KString& KString::operator= (const wchar_t* const other)
{
	int length = (int)wcslen(other);
	if(length)
	{
		if(count)
		{
			free(a_text);
			free(w_text);
		}

		w_text=(wchar_t*)malloc((length+1)*sizeof(wchar_t));
		wcscpy(w_text,other);

		count=length;

		length = WideCharToMultiByte(CP_ACP,0, other, -1, 0,0,0,0);
		if (length)
		{
			a_text = (char*)malloc(length);
			if (WideCharToMultiByte(CP_ACP,0, other, -1, a_text,length,0,0))
			{
				return *this;
			}else
			{
				free(a_text);
				free(w_text);
			}
		}else
		{
			free(w_text);
		}
	}else
	{
		if(count)
		{
			free(a_text);
			free(w_text);
		}
	}

	a_text=0;
	w_text=0;
	count=0;

	return *this;
}

const KString KString::operator+ (const KString& stringToAppend)
{
	return Append(stringToAppend);
}

const KString KString::operator+ (const wchar_t* const textToAppend)
{
	return Append(KString(textToAppend));
}

KString::operator const char*()
{
	return a_text;
}

KString::operator const wchar_t*()
{
	return w_text;
}

const char KString::operator[](const int index)
{
	if((0<=index) && (index<=(count-1)))
		return a_text[index];
	return -1;
}

KString KString::Append(const KString &otherString)
{
	if(otherString.count)
	{
		KString result;
		int length=otherString.count;
		result.w_text=(wchar_t*)malloc((length+count+1)*sizeof(wchar_t));
		result.a_text=(char*)malloc(length+count+1);
		if(count) // this string is not empty!
		{
			wcscpy(result.w_text,w_text);
			wcsncat(result.w_text,otherString.w_text,length);

			strcpy(result.a_text,a_text);
			strncat(result.a_text,otherString.a_text,length);
		}else
		{
			wcscpy(result.w_text,otherString.w_text);
			strcpy(result.a_text,otherString.a_text);
		}

		result.count=length+count;
		return result;
	}else
	{
		return *this;
	}
}

KString KString::SubString(int start,int end)
{
	if((0<=start) && (start<=(count-1)))
	{
		if((start<end) && (end<=(count-1)))
		{
			int size=(end-start)+1;
			wchar_t* buf=(wchar_t*)malloc((size+1)*sizeof(wchar_t));
			wcsncpy(buf,&w_text[start],size);
			buf[size]=0;

			KString result(buf);
			free(buf);
			return result;
		}
	}
	return KString();
}

bool KString::EqualsIgnoreCase(KString &otherString)
{
	if(wcscmp(w_text,otherString.w_text)==0)
		return true;
	return false;
}

bool KString::StartsWithChar(wchar_t character)
{
	if(w_text[0]==character)
		return true;
	return false;
}

bool KString::StartsWithChar(char character)
{
	if(a_text[0]==character)
		return true;
	return false;
}

bool KString::EndsWithChar(wchar_t character)
{
	if(w_text[count-1]==character)
		return true;
	return false;
}

bool KString::EndsWithChar(char character)
{
	if(a_text[count-1]==character)
		return true;
	return false;
}

bool KString::IsQuotedString()
{
	if(count>1)
	{
		if(StartsWithChar(L'\"') && EndsWithChar(L'\"'))
			return true;
	}
	return false;
}

wchar_t KString::GetCharAt(int index)
{
	if((0<=index) && (index<=(count-1)))
		return w_text[index];
	return -1;
}

int KString::GetLength()
{
	return count;
}

bool KString::IsEmpty()
{
	if(count)
		return false;
	return true;
}

int KString::GetIntValue()
{
	if(IsEmpty())
		return 0;
	return _wtoi(w_text);
}

KString::~KString()
{
	if(count)
	{
		free(a_text);
		free(w_text);
	}
}

